% Text Mining with MATLAB 2nd Edition
% www.http://www.textmininglab.net/

% CHAPTER 5: Reading and Writing Files

disp('Example 5.1 ------------------------------------------------------');
clear % deletes all variables in the workspace                      %(5.1a)
a = [1 2; 3 4]; % creates some variables                            %(5.1b)
b = {'Hello','World','!'}; c.f1 = "Hello"; c.f2 = "World!";
save % saves the workspace into default file matlab.mat             %(5.1c)
clear % deletes all variables in the workspace                      %(5.1d)
load % restores the workspace from default file matlab.mat          %(5.1e)
whos                                                                %(5.1f)
pause;

disp('Example 5.2 ------------------------------------------------------');
% save all variables into the file allworkspace.mat                 %(5.2a)
save('allworkspace'); 
% save only variables b and c into the file somevariables.mat       %(5.2b)
save('somevariables','b','c'); 
ls                                                                  %(5.2c)
clear % deletes all variables in the workspace                      %(5.2d)
% loads only variable a from the file allworkspace.mat              %(5.2e)
load('allworkspace','a'); 
% loads variables b and c from the file somevariables.mat           %(5.2f)
load('somevariables'); 
pause;

disp('Example 5.3 ------------------------------------------------------');
save allworkspace                                                   %(5.3a)
save somevariables b c                                              %(5.3b)
load allworkspace a                                                 %(5.3c)
load somevariables                                                  %(5.3d)
pause;

disp('Example 5.4 ------------------------------------------------------');
save somevariables a -append                                        %(5.4a)
whos -file somevariables                                            %(5.4b)
pause;

disp('Example 5.5 ------------------------------------------------------');
clear; load somevariables -regexp '[^a]'                            %(5.5a)
whos                                                                %(5.5b)
pause;

disp('Example 5.6 ------------------------------------------------------');
save structfile -struct c f1 f2                                     %(5.6a)
whos -file structfile                                               %(5.6b)
pause;

disp('Example 5.7 ------------------------------------------------------');
newstructure = load('structfile')                                    %(5.7)
pause;

disp('Example 5.8 ------------------------------------------------------');
clear; load allworkspace                                            %(5.8a)
save matrix.txt a -ascii                                            %(5.8b)
pause;

disp('Example 5.9 ------------------------------------------------------');
load matrix.txt                                                     %(5.9a)
whos                                                                %(5.9b)
pause;

disp('Example 5.10 -----------------------------------------------------');
a = load('matrix.txt')                                              %(5.10)
pause;

disp('Example 5.11 -----------------------------------------------------');
disp('THIS EXAMPLE IS NOT A MATLAB SCRIPT')                         %(5.11)
pause;

disp('Example 5.12 -----------------------------------------------------');
disp('THIS EXAMPLE IS NOT A MATLAB SCRIPT')                         %(5.12)
pause;

disp('Example 5.13 -----------------------------------------------------');
disp('THIS EXAMPLE IS NOT A MATLAB SCRIPT')                         %(5.13)
pause;

disp('Example 5.14 -----------------------------------------------------');
fprintf('this is an example on printing plain text')                %(5.14)
pause;

disp('Example 5.15 -----------------------------------------------------');
fprintf('this is an example\non ''escape characters''\n')           %(5.15)
pause;

disp('Example 5.16 -----------------------------------------------------');
disp('THIS EXAMPLE IS NOT A MATLAB SCRIPT')                         %(5.16)
pause;

disp('Example 5.17 -----------------------------------------------------');
% uses the identifier operator and integer conversion              %(5.17a)
fprintf('decimal: %1$d  octal: %1$o  hexadecimal: %1$x\n',110)
% uses the identifier operator and floating point conversion       %(5.17b)
fprintf('fixed point: %1$f  exponential: %1$e\n',110)
% uses the identifier operator and character conversion            %(5.17c)
fprintf('character: %1$c  string: %1$s\n',110)
pause;

disp('Example 5.18 -----------------------------------------------------');
fprintf('character: %1$c  string: %1$s\n',char(110))                %(5.18)
pause;

disp('Example 5.19 -----------------------------------------------------');
fprintf(' %+10.4f %+10.4f\n %010.4f %010.4f\n',pi,-1,2,-pi)         %(5.19)
pause;

disp('Example 5.20 -----------------------------------------------------');
disp('THIS EXAMPLE IS NOT A MATLAB SCRIPT')                         %(5.20)
pause;

disp('Example 5.21 -----------------------------------------------------');
% opens the file and gets the file identifier                      %(5.21a)
fid = fopen('matrix.txt','rt','native','US-ASCII');
[data,count] = fscanf(fid,'%e',[1,4]) % reads four data elements   %(5.21b)
fclose(fid); % closes the file                                     %(5.21c)
pause;

disp('Example 5.22 -----------------------------------------------------');
fid = fopen('matrix.txt','rt','native','US-ASCII');                %(5.22a)
[data,count] = fscanf(fid,'%c',inf) % reads characters from file   %(5.22b)
fclose(fid);                                                       %(5.22c)
pause;

disp('Example 5.23 -----------------------------------------------------');
mtx = rand(3,4); % creates a random-valued matrix of size 3x4      %(5.23a)
csvwrite('mtx.csv',mtx); % saves the matrix into a csv file        %(5.23b)
type mtx.csv                                                       %(5.23c)
newmtx = csvread('mtx.csv') % loads the matrix into a new variable %(5.23d)
pause;

disp('Example 5.24 -----------------------------------------------------');
% reproduces the 2nd line of file sample_file.csv                  %(5.24a)
file_line = "Peter,625-7725,""Feb, 27th""";
% preprocesses the line contents                                   %(5.24b)
temp = replace(replace(file_line,',,',', ,'),',,',', ,') + ","
% extracts the fields                                              %(5.24c)
fields = regexp(temp,' *"(.*?)" *,|([^"].*?),','tokens')
pause;

disp('Example 5.25 -----------------------------------------------------');
% row-column-value data sample from figure 5.3                     %(5.25a)
rcvdata = [1,2,250;6,3,-110;7,1,152;7,4,49;9,6,0]
% creates a sparse matrix variable from row-column-value data      %(5.25b)
mtx = spconvert(rcvdata)
whos mtx                                                           %(5.25c)
pause;

disp('Example 5.26 -----------------------------------------------------');
% appends an imaginary part to rcvdata from (5.25a)                %(5.26a)
cdata = [rcvdata,[100;220;-150;100;0]];
% creates a complex sparse matrix variable                         %(5.26b)
cmtx = spconvert(cdata)
whos cmtx                                                          %(5.26c)
pause;

disp('Example 5.27 -----------------------------------------------------');
fmtx = full(mtx)                                                   %(5.27a)
smtx = sparse(fmtx)                                                %(5.27b)
whos fmtx smtx                                                     %(5.27c)
pause;

disp('Example 5.28 -----------------------------------------------------');
% gets row, column and value for all non-zero entries              %(5.28a)
[r,c,v] = find(smtx);
% concatenates row, column and value vectors                       %(5.28b)
back2rcv = [r,c,v];
% appends and additional entry for making explicit the matrix size %(5.28c)
back2rcv = [back2rcv;size(smtx),0]
pause;

disp('Example 5.29 -----------------------------------------------------');
disp('THIS EXAMPLE IS NOT A MATLAB SCRIPT')                         %(5.29)
pause;

disp('Example 5.30 -----------------------------------------------------');
data = {'name','age';'mark','35';'beth','26';'peter','39'}; 	   %(5.30a)
[status,message] = xlswrite('sample_file.xls',data) 	           %(5.30b)
pause;

disp('Example 5.31 -----------------------------------------------------');
newdata = rand(3,5);                                               %(5.31a)
[status,message] = xlswrite('sample_file.xls',newdata,2);          %(5.31b)
pause;

disp('Example 5.32 -----------------------------------------------------');
[status,message] = xlswrite('sample_file.xls',ones(2,3),2,'C2:E3');	%(5.32)
pause;

disp('Example 5.33 -----------------------------------------------------');
disp('THIS EXAMPLE IS NOT A MATLAB SCRIPT')                         %(5.33)
pause;

disp('Example 5.34 -----------------------------------------------------');
[isexcel,worksheets,format] = xlsfinfo('sample_file.xls')           %(5.34)
pause;

disp('Example 5.35 -----------------------------------------------------');
disp('THIS EXAMPLE IS NOT A MATLAB SCRIPT')                         %(5.35)
pause;

disp('Example 5.36 -----------------------------------------------------');
[num,txt,rawdata] = xlsread('sample_file.xls',1,'A2:B4')            %(5.36)
pause;

disp('Example 5.37 -----------------------------------------------------');
[num,txt,rawdata] = xlsread('sample_file.xls',1,'A2:B4','basic');  %(5.37a)
rawdata                                                            %(5.37b)
pause;

disp('Example 5.38 -----------------------------------------------------');
disp('THIS EXAMPLE IS NOT A MATLAB SCRIPT')                         %(5.38)
pause;

disp('Example 5.39 -----------------------------------------------------');
language = {'fr','en'};                                            %(5.39a)
name = {'Ren Descartes','Winston Churchill'};
data = {'Mais la volont est tellement libre de sa nature, qu''elle ne peut jamais tre contrainte.','You have enemies? Good. That means you''ve stood up for something in your life.'};
cite = struct('lang',language,'person',name,'statement',data)      %(5.39b)
pause;

disp('Example 5.40 -----------------------------------------------------');
fields = fieldnames(cite); % gets the names of the structure fields %(5.40)
fid = fopen('sample_file.xml','wt','native','UTF-8'); % creates the file 
fprintf(fid,'<?xml version="1.0" encoding="UTF-8" ?>\n'); % first line 
fprintf(fid,'<document id="1">\n'); % writes opening tag for document 1
for k=1:length(cite) % cite loop: writes one cite element per iteration
    % gets and writes cite's attribute and value within the opening tag
    atr = fields{1}; val = getfield(cite(k),atr);
    fprintf(fid,'  <cite id="%d" %s="%s">\n',k,atr,val);
    for n = 2:3 % cite elements loop: gets and writes person and statement 
        tag = fields{n}; txt = getfield(cite(k),fields{n});
        fprintf(fid,'  <%1$s>%2$s</%1$s>\n',tag,txt);
    end;
    fprintf(fid,'  </cite>\n'); % writes closing tag for cite 
end;
fprintf(fid,'</document>\n'); % writes closing tag for document 1
fclose(fid); % closes the file  
pause;

disp('Example 5.41 -----------------------------------------------------');
% reads the file and saves it into a string variable               %(5.41a)
fid = fopen('sample_file.xml','rt','native','UTF-8');
datastring = fscanf(fid,'%c',inf);  
fclose(fid);
% gets opening tag and content for each cite element in the file   %(5.41b)
data = regexp(datastring,'(<cite.*?>)(.*?)</cite>','tokens'); 
% cite loop: extracts attributes and elements for each cite
for k=1:1:length(data)
    temp = regexp(data{k}{1},'id="(.*?)"','tokens'); % gets the id
    index = str2num(temp{1}{1}); % converts the id into a numeric value
    lang = regexp(data{k}{1},'lang="(.*?)"','tokens'); % gets the language
    % gets the person and statement elements
    pers = regexp(data{k}{2},'<person>(.*?)</person>','tokens');  
    stat = regexp(data{k}{2},'<statement>(.*?)</statement>','tokens');
    % assigns values to corresponding fields and elements in structure cite 
    cite(index) = struct('lang',lang{1}{1},'person',pers{1}{1},...
    'statement',stat{1}{1});
end;
pause;

disp('Example 5.42 -----------------------------------------------------');
% creates the sample .html file from Figure 5.7                     %(5.42)
htmldata = ["<!DOCTYPE HTML>","<html>","<head>", ...
            "<title>Text Mining with MATLAB</title>",...
            "<meta charset=""utf-8"" />","</head>",...
            "<body>","<h2>This is a sample .html file</h2>",...
            "</body>","</html>"];
fid = fopen('sample_file.html','wt','native','UTF-8');
for k=1:length(htmldata)
    fprintf(fid,'%s\n',htmldata(k));
end
fclose(fid);
pause;

disp('Example 5.43 -----------------------------------------------------');
htmldata = evalc('type(''sample_file.html'')'); % reads the file   %(5.43a)
tree = htmlTree(htmldata) % parses the html content                %(5.43b)
findElement(tree,'title') % extracts the 'TITLE' element           %(5.43c)
findElement(tree,'body') % extracts the 'BODY' element             %(5.43d)
% extracts the 'CHARSET' attribute of the 'META' element           %(5.43e)
getAttribute(findElement(tree,'meta'),'charset')
strip(extractHTMLText(htmldata)) % extracts all text elements      %(5.43f)
pause;

disp('Example 5.44 -----------------------------------------------------');
% extracts text elements directly from a formatted file             %(5.44)
strip(extractFileText('sample_file.html'))
pause;

disp('Example 5.45 -----------------------------------------------------');
what                                                                %(5.45)
pause;

disp('Example 5.46 -----------------------------------------------------');
matlabfiles = what                                                 %(5.46a)
matlabfiles.m                                                      %(5.46b)
pause;

disp('Example 5.47 -----------------------------------------------------');
type matrix.txt                                                     %(5.47)
pause;

disp('Example 5.48 -----------------------------------------------------');
%output = type('matrix.txt')                                        %(5.48)
% uncomment and execute example (5.45) to see the error message
pause;

disp('Example 5.49 -----------------------------------------------------');
output = evalc('type(''matrix.txt'')')                              %(5.49)
pause;

disp('Example 5.50 -----------------------------------------------------');
disp('THIS EXAMPLE IS NOT A MATLAB SCRIPT')                %(5.50a,b,c,d,e)
pause;

disp('Example 5.51 -----------------------------------------------------');
files = dir('sample_file.*'); filelist = {files.name}';            %(5.51a)
filelist = cellstr(ls('sample_file.*'));                           %(5.51b)
pause;

disp('Example 5.52 -----------------------------------------------------');
[select,stat]=listdlg('ListString',filelist,'ListSize',[160,100]);  %(5.52)
pause;

disp('Example 5.53 -----------------------------------------------------');
disp('THIS EXAMPLE IS NOT A MATLAB SCRIPT')                    %(5.53a,b,c)
pause;
 
disp('Example 5.54 -----------------------------------------------------');
filter = {'sample_file.*','Files created in Chapter 5';...          %(5.54)  
          '*.doc;*.xls','Microsoft Application''s files';...
          '*.txt','Plain text files';'*.*','All files'};
pause;

disp('Example 5.55 -----------------------------------------------------');
uiimport('mtx.csv')                                                 %(5.55)
pause;

disp('Example 5.56 -----------------------------------------------------');
% specifies the files to be included in the datastore              %(5.56a)
files = ["sample_file.html","sample_file.txt","sample_file.*v"];
% creates the datastore object                                     %(5.56b)
fds = fileDatastore(files,'ReadFcn',@extractFileText);
% loops over the files in the datastore to read their contents     %(5.56c)
strarray = [];
while hasdata(fds)
    textData = read(fds); % reads the content of a file
    strarray = [strarray;textData]; % copies content into string array
end
% displays the contents                                            %(5.56d)
for k=1:length(strarray)
    fname = split(fds.Files{k},'\');
    fname = fname{end};
    fcontent = strip(replace(strarray(k),newline,''));
    fprintf('\n*** %s ***\n%s\n',fname,fcontent); 
end
pause;
